### R code for Chapter 7.1.2
### Last update: 18/08/2014
### Last revised: 12/02/2021

###################################################
### Set working directory and load packages
###################################################
require(INLA)
inla.setOption(scale.model.default=TRUE)

require(splancs)
require(sf)
require(fields)
require(maptools)
require(lattice)
require(abind)
require(here)
remove(list=ls())
my.dir <- paste(here(),"/LBWGeorgia_data/",sep="")

download.file('https://sites.google.com/a/r-inla.org/stbook/LBWGeorgia_data.zip?attredirects=0&d=1', 'LBWG.zip')
unzip('LBWG.zip')

###################################################
### Code for Section 7.1.1
###################################################
# You neeed a folder called "LBW in Georgia" inside your working directory (my.dir)
# with the data downloaded from
# https://sites.google.com/a/r-inla.org/stbook/datasets

data <- read.csv(paste(my.dir,"data.final.csv",sep=""))
georgia <- sf::st_read(paste(my.dir,"co13_d00.shp",sep=""))

# Remove duplicates (area 88 and 94)
names <- georgia$NAME
georgia <- georgia[-c(99,100,105,137),]
data.georgia <- st_drop_geometry(georgia)

low.vector <- as.vector(as.matrix(data[,2:12]))#by column
E.vector <- as.vector(as.matrix(data[,13:23]))#by column
year <- numeric(0)
for(i in 1:11){ 
  year<- append(year,rep(i,dim(data)[1]))
}
county <- as.factor(rep(data[,1],11))

data <- data.frame(y= low.vector, E= E.vector, ID.area=as.numeric(county), ID.area1=as.numeric(county), year=year,
                  ID.year = year, ID.year1=year, ID.area.year = seq(1,length(county)))

# Spatial graph
Georgia.adj <- paste(my.dir,"Georgia.graph",sep="")
# Temporal graph
Temp.adj <- paste(my.dir,"Temp.graph",sep="")

formula.ST1 <- y ~  f(ID.area,model="bym",graph=Georgia.adj) +f(ID.year,model="rw2") + f(ID.year1,model="iid")
lcs <- inla.make.lincombs(ID.year = diag(11),  ID.year1 = diag(11))

model.ST1 <- inla(formula.ST1,family="poisson",data=data,E=E, 
                  control.predictor=list(compute=TRUE),
                  lincomb=lcs)

temporal.CAR <- lapply(model.ST1$marginals.random$ID.year, 
                     function(X){
                       marg <- inla.tmarginal(function(x) exp(x), X)
                       inla.emarginal(mean, marg)
                     })

temporal.IID <- lapply(model.ST1$marginals.random$ID.year1, 
                     function(X){
                       marg <- inla.tmarginal(function(x) exp(x), X)
                       inla.emarginal(mean, marg)
                     })

# *** Code for Figure 7.3
plot(seq(1,11),seq(0.9,1.1,length=11),type="n",xlab="t",ylab=expression(exp(phi[t])))
lines(unlist(temporal.IID))
lines(unlist(temporal.CAR),lty=2)
abline(h=1,lty=1)
# ***

###################################################
### Code for Section 7.1.2 (run the code for Section 7.1.1 first) 
###################################################
#--- Type I interaction ---#
formula.intI<- y ~ + f(ID.area,model="bym", graph=Georgia.adj) +
    f(ID.year,model="rw2") + f(ID.year1,model="iid") + f(ID.area.year,model="iid")

mod.intI <- inla(formula.intI,family="poisson",data=data,E=E, 
                 control.predictor=list(compute=TRUE), 
                 control.compute=list(dic=TRUE,cpo=TRUE))
print(summary(mod.intI))
#--- Type II interaction ---#
ID.area.int <- data$ID.area
ID.year.int <- data$ID.year
formula.intII<- y ~ f(ID.area,model="bym",graph=Georgia.adj) +
  f(ID.year,model="rw2") + f(ID.year1,model="iid") +
  f(ID.area.int,model="iid", group=ID.year.int,control.group=list(model="rw2")) 

mod.intII <- inla(formula.intII,family="poisson",data=data,E=E, 
                  control.predictor=list(compute=TRUE), 
                  control.compute=list(dic=TRUE,cpo=TRUE))
print(summary(mod.intII))

#--- Type III interaction ---#
formula.intIII<- y ~  f(ID.area,model="bym",graph=Georgia.adj) +
  f(ID.year,model="rw2") + 
  f(ID.year1,model="iid") +
  f(ID.year.int,model="iid", group=ID.area.int,control.group=list(model="besag", graph=Georgia.adj))

mod.intIII <- inla(formula.intIII,family="poisson",data=data,E=E, 
                   control.predictor=list(compute=TRUE), 
                   control.compute=list(dic=TRUE,cpo=TRUE))
print(summary(mod.intIII))
#--- Type IV interaction ---#
formula.intIV<- y ~ f(ID.area,model="bym",graph=Georgia.adj) +
  f(ID.year,model="rw2") +
  f(ID.year1,model="iid") + 
  f(ID.area.int,model="besag", graph=Georgia.adj,group=ID.year.int,control.group=list(model="rw2"))

mod.intIV <- inla(formula.intIV,family="poisson",data=data,E=E, 
                  control.predictor=list(compute=TRUE), 
                  control.compute=list(dic=TRUE,cpo=TRUE))
print(summary(mod.intIV))
delta.intI <- data.frame(delta=mod.intI$summary.random$ID.area.year[,2],year=data$ID.year,ID.area=data$ID.area)
delta.intI.matrix <- matrix(delta.intI[,1], 159,11,byrow=FALSE)
rownames(delta.intI.matrix)<- delta.intI[1:159,3]

delta.intII <- data.frame(delta=mod.intII$summary.random$ID.area.int[,2],year=data$ID.year,ID.area=data$ID.area)
delta.intII.matrix <- matrix(delta.intII[,1], 159,11,byrow=FALSE)
rownames(delta.intII.matrix)<- delta.intII[1:159,3]

delta.intIII <- data.frame(delta=mod.intIII$summary.random$ID.year.int[,2],year=data$ID.year,ID.area=data$ID.area)
delta.intIII.matrix <- matrix(delta.intIII[,1], 159,11,byrow=FALSE)
rownames(delta.intIII.matrix)<- delta.intIII[1:159,3]

delta.intIV <- data.frame(delta=mod.intIV$summary.random$ID.area.int[,2],year=data$ID.year,ID.area=data$ID.area)
delta.intIV.matrix <- matrix(delta.intIV[,1], 159,11,byrow=FALSE)
rownames(delta.intIV.matrix)<- delta.intIV[1:159,3]

# Check the absence of spatial trend for (intI)
cutoff.interaction <- c(-1,-0.01,0.01,1)
delta.intI.factor <- data.frame(NAME=data.georgia$NAME)
for(i in 1:11){
  delta.factor.temp <- cut(delta.intI.matrix[,i],breaks=cutoff.interaction,include.lowest=TRUE) 
  delta.intI.factor <- cbind(delta.intI.factor,delta.factor.temp)
}
colnames(delta.intI.factor)<- c("NAME",seq(2000,2010))

# Check the absence of spatial trend for (intII)
delta.intII.factor <- data.frame(NAME=data.georgia$NAME)
for(i in 1:11){
  delta.factor.temp <- cut(delta.intII.matrix[,i],breaks=cutoff.interaction,include.lowest=TRUE) 
  delta.intII.factor <- cbind(delta.intII.factor,delta.factor.temp)
}
colnames(delta.intII.factor)<- c("NAME",seq(2000,2010))

# Check the absence of spatial trend (intIII)
delta.intIII.factor <- data.frame(NAME=data.georgia$NAME)
for(i in 1:11){
  delta.factor.temp <- cut(delta.intIII.matrix[,i],breaks=cutoff.interaction,include.lowest=TRUE) 
  delta.intIII.factor <- cbind(delta.intIII.factor,delta.factor.temp)
}
colnames(delta.intIII.factor)<- c("NAME",seq(2000,2010))

# Check the Spatial trend (intIV)
delta.intIV.factor <- data.frame(NAME=data.georgia$NAME)
for(i in 1:11){
  delta.factor.temp <- cut(delta.intIV.matrix[,i],breaks=cutoff.interaction,include.lowest=TRUE) 
  delta.intIV.factor <- cbind(delta.intIV.factor,delta.factor.temp)
}
colnames(delta.intIV.factor)<- c("NAME",seq(2000,2010))
